#include <iostream>
#include <my_global.h>
#include <mysql.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <termios.h>
#include <sys/signal.h>
#include <unistd.h>

#define SERIAL_PORT "/dev/ttyS0"
#define BAUDRATE B9600
#define PRESSED_QUERY "UPDATE btn_state SET state=true"
#define NOTPRESSED_QUERY "UPDATE btn_state SET state=false"

void setupdb(MYSQL * conn);
int initSerial();
void signal_handler_IO (int status);
int getNewDataDB(MYSQL * conn, int & state);

int newData = 0;

using namespace std;

int main(int argc, char * argv[]) {
    MYSQL * conn = mysql_init(NULL);
    if(!mysql_real_connect(conn, "localhost", "grzegorz", "password1", "serial", 0, NULL, 0)){
        printf("Error %u: %s\n", mysql_errno(conn), mysql_error(conn));
	exit(1);
    }
    if (argc==2) if (strcmp(argv[1], "setup")==0) setupdb(conn);
    int fd = initSerial();
    while(1) {
	if(newData){
	    newData = 0;
	    char buf[1];
	    read(fd, buf, 1);
	    cout << buf[0] << endl;
	    if (mysql_query(conn, buf[0]=='P' ? PRESSED_QUERY : NOTPRESSED_QUERY)) {
		printf("Error %u: %s\n", mysql_errno(conn), mysql_error(conn));
		exit(1);
	    }
	}
	int state;
	if(getNewDataDB(conn, state)){
	    char buf[] = {'a'};
	    if(!state) buf[0] = 'b';
	    write(fd, buf, 1);
	    newData = 0;
	}
	usleep(100000);
    }
    mysql_close(conn);
    return 0;
}

void setupdb(MYSQL * conn) {
    mysql_query(conn, "drop table btn_state");
    mysql_query(conn, "drop table led_state");
    if (mysql_query(conn, "create table btn_state (state BOOL)")) {
	printf("Error %u: %s\n", mysql_errno(conn), mysql_error(conn));
	exit(1);
    }
    if (mysql_query(conn, "insert into  btn_state (state) values (false)")) {
	printf("Error %u: %s\n", mysql_errno(conn), mysql_error(conn));
	exit(1);
    }
    if (mysql_query(conn, "create table led_state (state BOOL, new BOOL)")) {
	printf("Error %u: %s\n", mysql_errno(conn), mysql_error(conn));
	exit(1);
    }
    if (mysql_query(conn, "insert into  led_state (state, new) values (false, false)")) {
	printf("Error %u: %s\n", mysql_errno(conn), mysql_error(conn));
	exit(1);
    }
    cout << "Tables created successfully." << endl;
}

int initSerial() {
    int fd = open(SERIAL_PORT, O_RDWR | O_NOCTTY);
    struct sigaction saio;
    saio.sa_handler = signal_handler_IO;
    sigemptyset(&saio.sa_mask);
    saio.sa_flags = 0;
    sigaction(SIGIO, &saio, NULL);
    fcntl(fd, F_SETOWN, getpid());
    fcntl(fd, F_SETFL, FASYNC);
    struct termios newtio;
    bzero(&newtio, sizeof(newtio));
    newtio.c_cflag = BAUDRATE | CS8 | CLOCAL | CREAD;
    newtio.c_iflag = 0;
    newtio.c_oflag = 0;
    newtio.c_lflag = 0;
    newtio.c_cc[VTIME] = 0;
    newtio.c_cc[VMIN] = 1;
    tcflush(fd, TCIFLUSH);
    tcsetattr(fd, TCSANOW, &newtio);
    return fd;
}

void signal_handler_IO(int status) {
    newData = 1;
}

int getNewDataDB(MYSQL * conn, int & state) {
    mysql_query(conn, "SELECT state, new FROM led_state");
    MYSQL_RES * result = mysql_store_result(conn);
    MYSQL_ROW row = mysql_fetch_row(result);
    int newState = atoi(row[1]);
    mysql_free_result(result);
    if(newState) {
	state = atoi(row[0]);
	if (mysql_query(conn, "UPDATE led_state SET new=false")) {
	    printf("Error %u: %s\n", mysql_errno(conn), mysql_error(conn));
	    exit(1);
	}
	if(state) cout << "LED ON" << endl; else cout << "LED OFF" << endl;
    }
    return newState;
}
